@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* overmaker.c
@@ -1491,21 +1490,21 @@ void legacyConfig () {
 
 #ifdef ADWS_GFB
   sprintf (buf,
-	   "else if (getConfigParameter_f (\"%s\", cfg, &v)) { %s(NULL, v); return 1; }",
+	   "else if (getConfigParameter_f (\"%s\", cfg, &v)) { %s(pp, v); return 1; }",
 	   "xov.ctl_biased_gfb", "fctl_biased_gfb");
   codeln (buf);
 #endif
 
 #ifdef TR_BIASED
   sprintf (buf,
-	   "else if (getConfigParameter_f (\"%s\", cfg, &v)) { %s(NULL, v); return 1; }",
+	   "else if (getConfigParameter_f (\"%s\", cfg, &v)) { %s(pp, v); return 1; }",
 	   "xov.ctl_biased", "fctl_biased");
   codeln (buf);
 #endif
 
 #ifdef ADWS_FAT_CTRL
   sprintf (buf,
-	   "else if (getConfigParameter_f (\"%s\", cfg, &v)) { %s(NULL, v); return 1; }",
+	   "else if (getConfigParameter_f (\"%s\", cfg, &v)) { %s(pp, v); return 1; }",
 	   "overdrive.character", "fctl_biased_fat");
   codeln (buf);
 #endif /* ADWS_FAT_CTRL */
