@@ -15,10 +15,11 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define _GNU_SOURCE
+
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
@@ -75,6 +76,7 @@ typedef struct {
 
   short suspend_ui_msg;
   short update_gui_now;
+  short update_pgm_now;
   short swap_instances;
 
   struct b_instance *inst;
@@ -82,14 +84,41 @@ typedef struct {
 } B3S;
 
 enum {
-  CMD_LOAD   = 0,
-  CMD_FREE   = 1,
+  CMD_FREE    = 0,
+  CMD_LOADPGM = 1,
+  CMD_LOADCFG = 2,
+  CMD_SAVEPGM = 3,
+  CMD_SAVECFG = 4,
+};
+
+struct worknfo {
+  int cmd;
+  int status;
+  char msg[1024];
 };
 
 /* main synth wrappers */
 
 const ConfigDoc *mainDoc () { return NULL;}
-int mainConfig (ConfigContext * cfg) { return 0; }
+
+int mainConfig (ConfigContext * cfg) {
+  if (strcasecmp (cfg->name, "midi.driver") == 0) {
+    return 1;
+  }
+  else if (strcasecmp (cfg->name, "midi.port") == 0) {
+    return 1;
+  }
+  else if (strcasecmp (cfg->name, "jack.connect") == 0) {
+    return 1;
+  }
+  else if (strcasecmp (cfg->name, "jack.out.left") == 0) {
+    return 1;
+  }
+  else if (strcasecmp (cfg->name, "jack.out.right") == 0) {
+    return 1;
+  }
+  return 0;
+}
 
 double SampleRateD = 48000.0;
 
@@ -198,6 +227,29 @@ static void rc_cb(int fnid, const char *key, const char *kv, unsigned char val,
   }
 }
 
+static void pgm_cb(int num, int pc, const char *name, void *arg) {
+  B3S* b3s = (B3S*)arg;
+  char tmp[256];
+  int pco = pc - b3s->inst->progs->MIDIControllerPgmOffset;
+#ifdef DEBUGPRINT
+      fprintf(stderr, "PGM CB %d %d %s\n",num, pc, name);
+#endif
+  LV2_Atom_Forge_Frame frame;
+  lv2_atom_forge_frame_time(&b3s->forge, 0);
+  lv2_atom_forge_blank(&b3s->forge, &frame, 1, b3s->uris.sb3_midipgm);
+
+  lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_cckey, 0);
+  lv2_atom_forge_int(&b3s->forge, pco);
+  lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_ccval, 0);
+  lv2_atom_forge_string(&b3s->forge, name, strlen(name));
+
+  formatProgram(&b3s->inst->progs->programmes[pc], tmp, 256);
+  lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_ccdsc, 0);
+  lv2_atom_forge_string(&b3s->forge, tmp, strlen(tmp));
+
+  lv2_atom_forge_pop(&b3s->forge, &frame);
+}
+
 static void mcc_cb(const char *fnname, const unsigned char chn, const unsigned char cc, const unsigned char flags, void *arg) {
   B3S* b3s = (B3S*)arg;
   char mmv[20];
@@ -234,7 +286,7 @@ void allocSynth(struct b_instance *inst) {
 
 
 /* LV2 -- state */
-static void rcsave_cb(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
+static void rcstate_cb(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
   char tmp[256];
   char **cfg = (char**)arg;
   if (fnid < 0) {
@@ -256,7 +308,24 @@ save(LV2_Handle                instance,
   B3S* b3s = (B3S*)instance;
 
   char *cfg = calloc(1, sizeof(char));
-  rc_loop_state(b3s->inst->state, rcsave_cb, (void*) &cfg);
+  rc_loop_state(b3s->inst->state, rcstate_cb, (void*) &cfg);
+
+  int i;
+  size_t rs = 0;
+  char *out = NULL;
+  FILE *x = open_memstream(&out, &rs);
+  for (i=0 ; i < 128; ++i) {
+    int pgmNr = i + b3s->inst->progs->MIDIControllerPgmOffset;
+    if (!(b3s->inst->progs->programmes[pgmNr].flags[0] & FL_INUSE)) {
+      continue;
+    }
+    fprintf(x, "P ");
+    writeProgramm(pgmNr, &b3s->inst->progs->programmes[pgmNr], " ", x);
+  }
+  fclose(x);
+
+  cfg = realloc(cfg, strlen(cfg) + strlen(out) +1);
+  strcat(cfg, out);
 
   store(handle, b3s->uris.sb3_state,
       cfg, strlen(cfg) + 1,
@@ -285,7 +354,7 @@ restore(LV2_Handle                  instance,
   }
 
   if (b3s->inst_offline) {
-    fprintf(stderr, "restore ignored. re-init in progress\n");
+    fprintf(stderr, "B3LV2: restore ignored. re-init in progress\n");
     return LV2_STATE_ERR_UNKNOWN;
   }
 
@@ -311,6 +380,12 @@ restore(LV2_Handle                  instance,
 #endif
       evaluateConfigKeyValue((void*)b3s->inst_offline, kv+2, val+1);
     }
+    else if(kv[0]=='P') {
+#ifdef DEBUGPRINT
+      printf("PGM '%s'\n", kv+2);
+#endif
+      loadProgrammeString(b3s->inst_offline->progs, kv+2);
+    }
     ts=te+1;
   }
 
@@ -340,6 +415,12 @@ restore(LV2_Handle                  instance,
   return LV2_STATE_SUCCESS;
 }
 
+static void rcsave_cb(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
+  if (fnid < 0) {
+    fprintf((FILE*)arg, "%s=%s\n", key, kv);
+  }
+}
+
 /* LV2 -- worker */
 static LV2_Worker_Status
 work(LV2_Handle                  instance,
@@ -349,19 +430,59 @@ work(LV2_Handle                  instance,
      const void*                 data)
 {
   B3S* b3s = (B3S*)instance;
+  FILE *x;
 
-  if (size != sizeof(int)) {
+  if (size != sizeof(struct worknfo)) {
     return LV2_WORKER_ERR_UNKNOWN;
   }
+  struct worknfo *w = (struct worknfo*) data;
 
-  switch(*((const int*)data)) {
-    case CMD_LOAD:
+  switch(w->cmd) {
+    case CMD_LOADPGM:
+      fprintf(stderr, "B3LV2: loading pgm file: %s\n", w->msg);
+      if (!(w->status=loadProgrammeFile(b3s->inst->progs, w->msg))) {
+	b3s->update_pgm_now = 1;
+      }
+      break;
+    case CMD_LOADCFG:
       if (b3s->inst_offline) {
-	fprintf(stderr, "restore ignored. re-init in progress\n");
+	fprintf(stderr, "B3LV2: restore ignored. re-init in progress\n");
 	return LV2_STATE_ERR_UNKNOWN;
       }
-      fprintf(stderr, "TODO -- load new cfg/pgm, re-init\n");
-
+      fprintf(stderr, "B3LV2: loading cfg file: %s\n", w->msg);
+      b3s->inst_offline = calloc(1, sizeof(struct b_instance));
+      allocSynth(b3s->inst_offline);
+      w->status = parseConfigurationFile (b3s->inst_offline, w->msg);
+      initSynth(b3s->inst_offline, SampleRateD);
+      break;
+    case CMD_SAVECFG:
+      x = fopen(w->msg, "w");
+      if (x) {
+	fprintf(x, "# setBfree config file\n# modificaions on top of default config\n");
+	rc_loop_state(b3s->inst->state, rcsave_cb, (void*) x);
+	fclose(x);
+	w->status = 0;
+      } else {
+	w->status = -1;
+      }
+      break;
+    case CMD_SAVEPGM:
+      x = fopen(w->msg, "w");
+      if (x) {
+	fprintf(x, "# setBfree midi program file\n");
+	int i;
+	for (i=0 ; i < 128; ++i) {
+	  int pgmNr = i + b3s->inst->progs->MIDIControllerPgmOffset;
+	  if (!(b3s->inst->progs->programmes[pgmNr].flags[0] & FL_INUSE)) {
+	    continue;
+	  }
+	  writeProgramm(pgmNr, &b3s->inst->progs->programmes[pgmNr], "\n    ", x);
+	}
+	fclose(x);
+	w->status = 0;
+      } else {
+	w->status = -1;
+      }
       break;
     case CMD_FREE:
 #ifdef DEBUGPRINT
@@ -372,27 +493,65 @@ work(LV2_Handle                  instance,
     break;
   }
 
+  respond(handle, sizeof(struct worknfo), data);
   return LV2_WORKER_SUCCESS;
 }
 
+static void forge_message_str(B3S *b3s, LV2_URID uri, const char *msg) {
+  LV2_Atom_Forge_Frame frame;
+  lv2_atom_forge_frame_time(&b3s->forge, 0);
+  lv2_atom_forge_blank(&b3s->forge, &frame, 1, uri);
+  lv2_atom_forge_property_head(&b3s->forge, b3s->uris.sb3_uimsg, 0);
+  lv2_atom_forge_string(&b3s->forge, msg, strlen(msg));
+  lv2_atom_forge_pop(&b3s->forge, &frame);
+}
+
 static LV2_Worker_Status
 work_response(LV2_Handle  instance,
               uint32_t    size,
               const void* data)
 {
   B3S* b3s = (B3S*)instance;
+  char tmp[1048];
 
-  if (size != sizeof(int)) {
+  if (size != sizeof(struct worknfo)) {
     return LV2_WORKER_ERR_UNKNOWN;
   }
 
-  switch(*((const int*)data)) {
-    case CMD_LOAD:
-      fprintf(stderr, "loaded new instance\n");
-      // TODO only if sucessful
+  struct worknfo *w = (struct worknfo*) data;
+
+  switch(w->cmd) {
+    case CMD_LOADCFG:
       b3s->swap_instances = 1;
+      if (w->status)
+	sprintf(tmp, "error loading CFG: '%s'", w->msg);
+      else
+	sprintf(tmp, "loaded CFG: '%s'", w->msg);
+      forge_message_str(b3s, b3s->uris.sb3_uimsg, tmp);
       break;
-    case CMD_FREE:
+    case CMD_LOADPGM:
+      if (w->status)
+	sprintf(tmp, "error loading PGM: '%s'", w->msg);
+      else
+	sprintf(tmp, "loaded PGM: '%s'", w->msg);
+      forge_message_str(b3s, b3s->uris.sb3_uimsg, tmp);
+      break;
+    case CMD_SAVEPGM:
+      if (w->status)
+	sprintf(tmp, "error saving PGM: '%s'", w->msg);
+      else
+	sprintf(tmp, "saved PGM: '%s'", w->msg);
+      forge_message_str(b3s, b3s->uris.sb3_uimsg, tmp);
+      break;
+    case CMD_SAVECFG:
+      if (w->status)
+	sprintf(tmp, "error saving CFG: '%s'", w->msg);
+      else
+	sprintf(tmp, "saved CFG: '%s'", w->msg);
+      forge_message_str(b3s, b3s->uris.sb3_uimsg, tmp);
+      break;
+    break;
+    default:
     break;
   }
   return LV2_WORKER_SUCCESS;
@@ -405,7 +564,8 @@ postrun (B3S* b3s)
 #ifdef DEBUGPRINT
     fprintf(stderr, "swap instances..\n");
 #endif
-    int d = CMD_FREE;
+    struct worknfo w;
+    w.cmd = CMD_FREE;
     /* swap engine instances */
     struct b_instance *old  = b3s->inst;
     b3s->inst = b3s->inst_offline;
@@ -416,12 +576,23 @@ postrun (B3S* b3s)
     /* hide midi-maps, stop possibly pending midi-bind process */
     forge_kvcontrolmessage(&b3s->forge, &b3s->uris, "special.midimap", (int32_t) 0);
 
-    b3s->schedule->schedule_work(b3s->schedule->handle, sizeof(int), &d);
+    b3s->schedule->schedule_work(b3s->schedule->handle, sizeof(struct worknfo), &w);
     b3s->update_gui_now = 1;
     b3s->swap_instances = 0;
   }
 }
 
+static void iowork(B3S* b3s, const LV2_Atom_Object* obj, int cmd) {
+  const LV2_Atom* name = NULL;
+  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &name, 0);
+  if (name) {
+    struct worknfo w;
+    w.cmd = cmd;
+    strncpy(w.msg, (char *)LV2_ATOM_BODY(name), 1024);
+    b3s->schedule->schedule_work(b3s->schedule->handle, sizeof(struct worknfo), &w);
+  }
+}
+
 /* main LV2 */
 
 static LV2_Handle
@@ -458,11 +629,11 @@ instantiate(const LV2_Descriptor*     descriptor,
 
   srand ((unsigned int) time (NULL));
   b3s->suspend_ui_msg = 1;
-  b3s->update_gui_now = 0;
   b3s->boffset = BUFFER_SIZE_SAMPLES;
 
   b3s->swap_instances = 0;
   b3s->update_gui_now = 0;
+  b3s->update_pgm_now = 0;
 
   b3s->inst = calloc(1, sizeof(struct b_instance));
   b3s->inst_offline = NULL;
@@ -547,6 +718,28 @@ run(LV2_Handle instance, uint32_t n_samples)
 	  if (key) {
 	    midi_uiassign_cc(b3s->inst->midicfg, (const char*)LV2_ATOM_BODY(key));
 	  }
+	} else if (obj->body.otype == b3s->uris.sb3_midipgm) {
+	  const LV2_Atom* key = NULL;
+	  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &key, 0);
+	  if (key) {
+	    installProgram(b3s->inst, ((LV2_Atom_Int*)key)->body);
+	  }
+	} else if (obj->body.otype == b3s->uris.sb3_midisavepgm) {
+	  const LV2_Atom* pgm = NULL;
+	  const LV2_Atom* name = NULL;
+	  lv2_atom_object_get(obj, b3s->uris.sb3_cckey, &pgm, b3s->uris.sb3_ccval, &name, 0);
+	  if (pgm && name) {
+	    saveProgramm(b3s->inst, (int) ((LV2_Atom_Int*)pgm)->body, (char*) LV2_ATOM_BODY(name), 0);
+	    b3s->update_pgm_now = 1;
+	  }
+	} else if (obj->body.otype == b3s->uris.sb3_loadpgm) {
+	  iowork(b3s, obj, CMD_LOADPGM);
+	} else if (obj->body.otype == b3s->uris.sb3_loadcfg) {
+	  iowork(b3s, obj, CMD_LOADCFG);
+	} else if (obj->body.otype == b3s->uris.sb3_savepgm) {
+	  iowork(b3s, obj, CMD_SAVEPGM);
+	} else if (obj->body.otype == b3s->uris.sb3_savecfg) {
+	  iowork(b3s, obj, CMD_SAVECFG);
 	} else if (obj->body.otype == b3s->uris.sb3_control) {
 	  b3s->suspend_ui_msg = 1;
 	  const LV2_Atom_Object* obj = (LV2_Atom_Object*)&ev->body;
@@ -566,9 +759,13 @@ run(LV2_Handle instance, uint32_t n_samples)
 
   if (b3s->update_gui_now) {
     b3s->update_gui_now = 0;
+    b3s->update_pgm_now = 1;
     b3s->suspend_ui_msg = 1;
     rc_loop_state(b3s->inst->state, rc_cb, b3s);
     b3s->suspend_ui_msg = 0;
+  } else if (b3s->update_pgm_now) {
+    b3s->update_pgm_now = 0;
+    loopProgammes(b3s->inst->progs, 1, pgm_cb, b3s);
   }
 
   /* synthesize [remaining] sound */
