@@ -16,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 //#define DEBUG_SPEED  // debug acceleration,deceleration
@@ -45,7 +44,7 @@ void initValues(struct b_whirl *w) {
   w->hnBreakPos=0;
   w->drBreakPos=0;
 
-  for (i=0; i<4; ++i) 
+  for (i=0; i<4; ++i)
     w->z[i] = 0;
 
   /* The current angle of rotating elements */
@@ -62,7 +61,7 @@ void initValues(struct b_whirl *w) {
  *  drum: fast:5.955 Hz, slow: 0.101 Hz (wrong?)
  *
  * alternate values:
- * http://www.dairiki.org/HammondWiki/LeslieRotationSpeed 
+ * http://www.dairiki.org/HammondWiki/LeslieRotationSpeed
  *  horn: fast: 400 RPM, slow: 48 RPM
  *  drum: fast: 342 RPM, slow: 40 RPM
  */
@@ -82,8 +81,8 @@ void initValues(struct b_whirl *w) {
   w->hornAcDc = w->drumAcDc = 0;
 
   /* angular speed - unit: radians / sample / (2*M_PI) */
-  w->hornIncrUI = 0; ///< current angular speed 
-  w->drumIncrUI = 0; ///< current angular speed
+  w->hornIncrGRD = 0; ///< current angular speed
+  w->drumIncrGRD = 0; ///< current angular speed
 
   w->hornTarget = 0; ///< target angular speed
   w->drumTarget = 0; ///< target angular speed
@@ -104,7 +103,7 @@ void initValues(struct b_whirl *w) {
   w->drumRadiusCm = 22.0;
 
   w->airSpeed = 340.0; /* Meters per second */
-  w->micDistCm= 2.0;   /* From mic to origin */
+  w->micDistCm= 42.0;  /* From mic to origin */
 
   w->drumSpacing[0] = 36.0;
   w->drumSpacing[1] = 39.0;
@@ -228,16 +227,16 @@ void useRevOption (struct b_whirl *w, int n) {
   w->hornTarget = w->revoptions[i].hornTarget;
   w->drumTarget = w->revoptions[i].drumTarget;
 
-  if (w->hornIncrUI < w->hornTarget) {
+  if (w->hornIncrGRD < w->hornTarget) {
     w->hornAcDc = 1;
   }
-  else if (w->hornTarget < w->hornIncrUI) {
+  else if (w->hornTarget < w->hornIncrGRD) {
     w->hornAcDc = -1;
   }
-  if (w->drumIncrUI < w->drumTarget) {
+  if (w->drumIncrGRD < w->drumTarget) {
     w->drumAcDc = 1;
   }
-  else if (w->drumTarget < w->drumIncrUI) {
+  else if (w->drumTarget < w->drumIncrGRD) {
     w->drumAcDc = -1;
   }
 
@@ -274,7 +273,7 @@ static void setRevOption (struct b_whirl *w,
   w->revoptions[i].drumTarget = drTgt;
 }
 
-static void computeRotationSpeeds (struct b_whirl *w) {
+void computeRotationSpeeds (struct b_whirl *w) {
   const double hfast = w->hornRPMfast / (w->SampleRateD * 60.0);
   const double hslow = w->hornRPMslow / (w->SampleRateD * 60.0);
   const double hstop = 0;
@@ -368,8 +367,6 @@ static void initTables (struct b_whirl *w) {
     b = drumRadiusSamples * sin (v);
     w->drFwdDispl[i] = sqrt ((a * a) + (b * b));
     w->drBwdDispl[DISPLC_SIZE - (i + 1)] = w->drFwdDispl[i];
-
-    a = (1.0 + cos (v)) / 2.0;	/* Goes from 1, 0.5, 0, 0.5, 1 */
   }
 
   w->hornPhase[0] = 0;
@@ -382,7 +379,7 @@ static void initTables (struct b_whirl *w) {
   w->hornPhase[5] = ((DISPLC_SIZE * 4) / 6);
 
   for (i = 0; i < 6; i++) {
-    w->hornSpacing[i] += hornRadiusSamples + 1.0;
+    w->hornSpacing[i] = w->hornSpacing[i] * w->SampleRateD / 22100.0 + hornRadiusSamples + 1.0;
   }
 
   w->drumPhase[0] = 0;
@@ -395,7 +392,7 @@ static void initTables (struct b_whirl *w) {
   w->drumPhase[5] = ((DISPLC_SIZE * 4) / 6);
 
   for (i = 0; i < 6; i++) {
-    w->drumSpacing[i] += drumRadiusSamples + 1.0;
+    w->drumSpacing[i] = w->drumSpacing[i] * w->SampleRateD / 22100.0 + drumRadiusSamples + 1.0;
   }
 
   setIIRFilter (w->drfL, w->lpT, w->lpF, w->lpQ, w->lpG, w->SampleRateD);
@@ -619,6 +616,12 @@ static void displayFilter (char * id, int T, float F, float Q, float G) {
   displayFilter ("Horn B", w->hbT, w->hbF, w->hbQ, w->hbG); \
 }
 
+#define UPDATE_D_FILTER { \
+  setIIRFilter (w->drfL, w->lpT, w->lpF, w->lpQ, w->lpG, w->SampleRateD); \
+  setIIRFilter (w->drfR, w->lpT, w->lpF, w->lpQ, w->lpG, w->SampleRateD); \
+  displayFilter ("Drum", w->lpT, w->lpF, w->lpQ, w->lpG); \
+}
+
 
 /*
  * Sets the type of the A horn IIR filter.
@@ -745,6 +748,29 @@ void fsetHornFilterBGain (struct b_whirl *w, float v) {
   UPDATE_B_FILTER;
 }
 
+void isetDrumFilterType (struct b_whirl *w, int v) {
+  w->lpT = (int) (v % 9);
+  UPDATE_D_FILTER;
+}
+
+void fsetDrumFilterFrequency (struct b_whirl *w, float v) {
+  if (v<20.0 || v> 8000.0) return;
+  w->lpF = v;
+  UPDATE_D_FILTER;
+}
+
+void fsetDrumFilterQ (struct b_whirl *w, float v) {
+  if (v<0.01 || v> 6.0) return;
+  w->lpQ = v;
+  UPDATE_D_FILTER;
+}
+
+void fsetDrumFilterGain (struct b_whirl *w, float v) {
+  if (v<-48.0 || v> 48.0) return;
+  w->lpG = v;
+  UPDATE_D_FILTER;
+}
+
 void setHornBreakPosition (void *d, unsigned char uc) {
   struct b_whirl *w = (struct b_whirl *) d;
   w->hnBreakPos = (double)uc/127.0;
@@ -1003,27 +1029,27 @@ void whirlProc2 (struct b_whirl *w,
   /* compute rotational speeds for this cycle */
   if (w->hornAcDc) {
     const double l = exp(-1.0/(w->SampleRateD / bufferLengthSamples * (w->hornAcDc>0? w->hornAcc : w->hornDec )));
-    w->hornIncrUI += (1-l) * (w->hornTarget - w->hornIncrUI);
+    w->hornIncrGRD += (1-l) * (w->hornTarget - w->hornIncrGRD);
 
-    if (fabs(w->hornTarget - w->hornIncrUI) < (1.0/360.0/w->SampleRateD) ) {
+    if (fabs(w->hornTarget - w->hornIncrGRD) < (1.0/360.0/w->SampleRateD) ) {
 #ifdef DEBUG_SPEED
       printf("AcDc Horn off\n");
 #endif
       w->hornAcDc = 0;
-      w->hornIncrUI = w->hornTarget;
+      w->hornIncrGRD = w->hornTarget;
     }
   }
 
   if (w->drumAcDc) {
     const double l = exp(-1.0/(w->SampleRateD / bufferLengthSamples * (w->drumAcDc>0? w->drumAcc: w->drumDec )));
-    w->drumIncrUI += (1-l) * (w->drumTarget - w->drumIncrUI);
+    w->drumIncrGRD += (1-l) * (w->drumTarget - w->drumIncrGRD);
 
-    if (fabs(w->drumTarget - w->drumIncrUI) < (1.0/360.0/w->SampleRateD)) {
+    if (fabs(w->drumTarget - w->drumIncrGRD) < (1.0/360.0/w->SampleRateD)) {
 #ifdef DEBUG_SPEED
       printf("ACDC Drum off\n");
 #endif
       w->drumAcDc = 0;
-      w->drumIncrUI = w->drumTarget;
+      w->drumIncrGRD = w->drumTarget;
     }
   }
 
@@ -1037,7 +1063,7 @@ void whirlProc2 (struct b_whirl *w,
    */
   if (w->hnBreakPos>0) {
     const double targetPos= w->hnBreakPos - floor(w->hnBreakPos);
-    if (!w->hornAcDc && w->hornIncrUI==0 && w->hornAngleGRD!=targetPos) {
+    if (!w->hornAcDc && w->hornIncrGRD==0 && w->hornAngleGRD!=targetPos) {
       w->hornAngleGRD += 1.0/400.0;
       w->hornAngleGRD = w->hornAngleGRD - floor(w->hornAngleGRD);
       if ((w->hornAngleGRD-targetPos) < (1.0/360.0)) w->hornAngleGRD=targetPos;
@@ -1045,7 +1071,7 @@ void whirlProc2 (struct b_whirl *w,
   }
   if (w->drBreakPos>0) {
     const double targetPos= w->drBreakPos - floor(w->drBreakPos);
-    if (!w->drumAcDc && w->drumIncrUI==0 && w->drumAngleGRD!=targetPos) {
+    if (!w->drumAcDc && w->drumIncrGRD==0 && w->drumAngleGRD!=targetPos) {
       w->drumAngleGRD += 1.0/400.0;
       w->drumAngleGRD = w->drumAngleGRD - floor(w->drumAngleGRD);
       if ((w->drumAngleGRD-targetPos) < (1.0/360.0)) w->drumAngleGRD=targetPos;
@@ -1059,36 +1085,36 @@ void whirlProc2 (struct b_whirl *w,
   unsigned int outpos = w->outpos;
 
   const float leakage = w->leakage;
-  const double hornIncrUI = w->hornIncrUI;
-  const double drumIncrUI = w->drumIncrUI;
+  const float hornLevel = w->hornLevel;
+  const double hornIncrGRD = w->hornIncrGRD;
+  const double drumIncrGRD = w->drumIncrGRD;
 
   const int * const hornPhase = w->hornPhase;
-  const int * const drumPhase = w->hornPhase;
+  const int * const drumPhase = w->drumPhase;
   const float * const hornSpacing = w->hornSpacing;
   const float * const drumSpacing = w->drumSpacing;
+  const float * const hnFwdDispl = w->hnFwdDispl;
+  const float * const hnBwdDispl = w->hnBwdDispl;
+  const float * const drFwdDispl = w->drFwdDispl;
+  const float * const drBwdDispl = w->drBwdDispl;
+
   float * const hafw = w->hafw;
   float * const hbfw = w->hbfw;
   float * const HLbuf = w->HLbuf;
   float * const HRbuf = w->HRbuf;
   float * const DLbuf = w->DLbuf;
   float * const DRbuf = w->DRbuf;
-  float * const hnFwdDispl = w->hnFwdDispl;
-  float * const hnBwdDispl = w->hnBwdDispl;
-  float * const drFwdDispl = w->drFwdDispl;
-  float * const drBwdDispl = w->drBwdDispl;
   float * const adx0 = w->adx0;
   float * const adx1 = w->adx1;
   float * const adx2 = w->adx2;
-  const float hornLevel = w->hornLevel;
   float * const drfL = w->drfL;
   float * const drfR = w->drfR;
   float * const z = w->z;
 
-  //const float * const * const bfw = w->bfw;
-  //const float * const * const bbw = w->bbw;
   const struct _bw * const bfw = w->bfw;
   const struct _bw * const bbw = w->bfw;
 
+
   int hornAngle = hornAngleGRD * DISPLC_SIZE;
   int drumAngle = drumAngleGRD * DISPLC_SIZE;
 
@@ -1098,7 +1124,7 @@ void whirlProc2 (struct b_whirl *w,
   if ((fgh++ % (int)(w->SampleRateD/128/5) ) ==0) {
     printf ("H:%.3f D:%.3f | HS:%.3f DS:%.3f [Hz]| HT:%.2f DT:%.2f [Hz]| %s %s\n",
 	(double)hornAngle/DISPLC_SIZE, (double)drumAngle/DISPLC_SIZE,
-	w->SampleRateD*(double)hornIncrUI, w->SampleRateD*(double)drumIncrUI,
+	w->SampleRateD*(double)hornIncrGRD, w->SampleRateD*(double)drumIncrGRD,
 	w->SampleRateD*(double)w->hornTarget, w->SampleRateD*(double)w->drumTarget,
 	acdc[w->hornAcDc+1], acdc[w->drumAcDc+1]
 	);
@@ -1117,16 +1143,7 @@ void whirlProc2 (struct b_whirl *w,
     float xx = x;
     float leak = 0;
 
-  //if (IS_DENORMAL(x)) fprintf(stderr,"DENORMAL x!\n");
-
-#define ANGFILTER(BW,DX,DI) {           \
-    xa  = BW[k].b[0] * x;                     \
-    xa += BW[k].b[1] * DX[(DI)];              \
-    xa += BW[k].b[2] * DX[((DI)+1) & AGMASK]; \
-    xa += BW[k].b[3] * DX[((DI)+2) & AGMASK]; \
-    xa += BW[k].b[4] * DX[((DI)+3) & AGMASK]; \
-    }
-
+    /* 0) define macros for code below */
 #define ADDHIST(DX,DI,XS) {      \
     DI = (DI + AGMASK) & AGMASK; \
     DX[DI] = XS;}
@@ -1135,7 +1152,11 @@ void whirlProc2 (struct b_whirl *w,
     k = ((hornAngle + hornPhase[(P)]) & DISPLC_MASK);       \
     t = hornSpacing[(P)] + DSP[k] + (float) outpos;         \
     r = floorf (t);                                         \
-    ANGFILTER(BW,DX,DI);                                    \
+    xa  = BW[k].b[0] * x;                                   \
+    xa += BW[k].b[1] * DX[(DI)];                            \
+    xa += BW[k].b[2] * DX[((DI)+1) & AGMASK];               \
+    xa += BW[k].b[3] * DX[((DI)+2) & AGMASK];               \
+    xa += BW[k].b[4] * DX[((DI)+3) & AGMASK];               \
     q = xa * (t - r);                                       \
     n = ((unsigned int) r) & BUF_MASK_SAMPLES;              \
     BUF[n] += xa - q;                                       \
@@ -1153,16 +1174,10 @@ void whirlProc2 (struct b_whirl *w,
     BUF[n] += q;}
 
     /* This is just a bum filter to take some high-end off. */
-#if 1
 #define FILTER_C(W0,W1,I) {           \
     float temp = x;                   \
     x = ((W0) * x) + ((W1) * z[(I)]); \
     z[(I)] = temp; }
-#else
-#define FILTER_C(W0,W1,I) {           \
-    x = ((W0) * x) + ((W1) * z[(I)]); \
-    z[(I)] = x;}
-#endif
 
 #define EQ_IIR(W,X,Y) {                                     \
     float temp = (X) - (W[a1] * W[z0]) - (W[a2] * W[z1]);   \
@@ -1170,16 +1185,6 @@ void whirlProc2 (struct b_whirl *w,
     W[z1] = W[z0]; \
     W[z0] = temp;}
 
-    /* 1) apply filters A,B -- horn-speaker characteristics
-     * intput: x
-     * output: x', leak
-     */
-
-    EQ_IIR(hafw, x, x);
-    EQ_IIR(hbfw, x, x);
-
-    leak = x * leakage;
-
 #ifdef HORN_COMB_FILTER
 
 #define COMB(WP,RP,BP,ES,FB,X) { \
@@ -1188,6 +1193,19 @@ void whirlProc2 (struct b_whirl *w,
     if ((RP) == (ES)) RP = BP;   \
     if ((WP) == (ES)) WP = BP;}
 
+#endif
+
+    /* 1) apply filters A,B -- horn-speaker characteristics
+     * input: x
+     * output: x', leak
+     */
+
+    EQ_IIR(hafw, x, x);
+    EQ_IIR(hbfw, x, x);
+
+    leak = x * leakage;
+
+#ifdef HORN_COMB_FILTER
     /* only causes hiss-noise - in particular on 'E-4,F-4' ~660Hz
      * no audible benefit to leslie effect so far, needs tweaking
      */
@@ -1196,7 +1214,7 @@ void whirlProc2 (struct b_whirl *w,
 #endif
 
     /* 2) now do doppler shift for the horn -- FM
-     * intput: x' (filtered x)
+     * input: x' (filtered x)
      * output: HLbuf, HRbuf, leak
      */
 
@@ -1223,11 +1241,11 @@ void whirlProc2 (struct b_whirl *w,
     ADDHIST(adx2, w->adi2, x);
 
     /* 1A) do doppler shift for drum (actually orig signal -- FM
-     * intput: x
+     * input: x
      * output: DLbuf, DRbuf
      */
 
-    x = xx; // x is used in macros, xx is the original input siganl
+    x = xx; // use original input signal ('x' was modified by horn filters)
 
     /* --- DRUM --- */
     DR_MOTION(0, DLbuf, drFwdDispl);
@@ -1254,17 +1272,17 @@ void whirlProc2 (struct b_whirl *w,
       float y;
       EQ_IIR(drfL, DLbuf[outpos], y);
       if (outL)
-	*outL++ = (float) (y + (hornLevel * HLbuf[outpos]) + leak);
+	*outL++ = y + hornLevel * HLbuf[outpos] + leak;
       if (outHL)
-	*outHL++ = (hornLevel * HLbuf[outpos]) + leak;
+	*outHL++ = hornLevel * HLbuf[outpos] + leak;
       if (outDL)
 	*outDL++ = y;
 
       EQ_IIR(drfR, DRbuf[outpos], y);
       if (outR)
-	*outR++ = (float) (y + (hornLevel * HRbuf[outpos]) + leak);
+	*outR++ =  y + hornLevel * HRbuf[outpos] + leak;
       if (outHR)
-	*outHR++ = (hornLevel * HRbuf[outpos]) + leak;
+	*outHR++ = hornLevel * HRbuf[outpos] + leak;
       if (outDR)
 	*outDR++ = y;
     }
@@ -1274,14 +1292,16 @@ void whirlProc2 (struct b_whirl *w,
     DLbuf[outpos] = 0.0;
     DRbuf[outpos] = 0.0;
 
+    /* rotate speakers */
+
     outpos = (outpos + 1) & BUF_MASK_SAMPLES;
 
-    hornAngleGRD = (hornAngleGRD + hornIncrUI);
-    hornAngleGRD = hornAngleGRD - floor(hornAngleGRD);
+    hornAngleGRD = (hornAngleGRD + hornIncrGRD);
+    hornAngleGRD = hornAngleGRD - floor(hornAngleGRD); // limit to [0..1]
     hornAngle = hornAngleGRD * DISPLC_SIZE;
 
-    drumAngleGRD = (drumAngleGRD + drumIncrUI);
-    drumAngleGRD = drumAngleGRD - floor(drumAngleGRD);
+    drumAngleGRD = (drumAngleGRD + drumIncrGRD);
+    drumAngleGRD = drumAngleGRD - floor(drumAngleGRD); // limit to [0..1]
     drumAngle = drumAngleGRD * DISPLC_SIZE;
   }
 
