@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -447,6 +446,8 @@ static void Usage (int configdoc) {
   "  -D, --noCC        do not load default CC map on startup\n"
   "  -h                Print short help text\n"
   "  -H, --help        Print complete help text with parameter list\n"
+  "  -M <filename>, --midnam <filename>\n"
+  "                    export current controller mapping to .midnam file\n"
   "  -p <filename>, --program <filename>\n"
   "                    Load alternate program file over default\n"
   "  -P, --noprogram   Do not read the default program file\n"
@@ -622,6 +623,7 @@ int main (int argc, char * argv []) {
     { "noconfig",   no_argument,       0, 'C' },
     { "dumpcc",     no_argument,       0, 'd' },
     { "noCC",       no_argument,       0, 'D' },
+    { "midnam",     required_argument, 0, 'M' },
     { "noprogram",  no_argument,       0, 'P' },
     { "randomize",  no_argument,       0, 'r' },
     { "version",    no_argument,       0, 'V' },
@@ -816,11 +818,13 @@ int main (int argc, char * argv []) {
 #ifdef HAVE_ASEQ
   pthread_t t_midi;
   if (!use_jack_midi) {
-    if (!aseq_open(midi_port))
-      k= pthread_create(&t_midi, NULL, aseq_run, &inst);
-
-    if (k != 0) {
-      fprintf (stderr, "%d : %s\n", k, "pthread_create : MIDIInReader thread");
+    if (!aseq_open(midi_port)) {
+      k = pthread_create(&t_midi, NULL, aseq_run, &inst);
+      if (k != 0) {
+	fprintf (stderr, "%d : %s\n", k, "pthread_create : MIDIInReader thread");
+	return (1);
+      }
+    } else {
       return (1);
     }
   }
