@@ -15,14 +15,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* pgmParser.h */
 #ifndef PGMPARSER_H
 #define PGMPARSER_H
 
+#include "memstream.h"
+
 extern int loadProgrammeFile (void *p, char * fileName);
+extern int loadProgrammeString (void *p, char * pdef);
 
 #endif /* PGMPARSER_H */
