@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -56,54 +55,6 @@
 #define FILE_BUFFER_SIZE 2048
 
 
-
-/* Flag bits used in the first field */
-
-#define FL_INUSE  0x0001	/* Record is in use */
-
-#define FL_DRAWBR 0x0002	/* Set drawbars */
-
-#define FL_ATKENV 0x0004	/* Attack envelope */
-#define FL_ATKCKL 0x0008	/* Attack Click level */
-#define FL_ATKCKD 0x0010	/* Attack click duration */
-
-#define FL_RLSENV 0x0020	/* Release envelope */
-#define FL_RLSCKL 0x0040	/* Release level */
-#define FL_RLSCKD 0x0080	/* Release duration */
-
-#define FL_SCANNR 0x0100	/* Vibrato scanner modulation depth */
-
-#define FL_PRCENA 0x0200	/* Percussion on/off */
-#define FL_PRCVOL 0x0400	/* Percussion soft/normal */
-#define FL_PRCSPD 0x0800	/* Percussion slow/fast */
-#define FL_PRCHRM 0x1000	/* Percussion 2nd/3rd */
-
-#define FL_OVRSEL 0x2000	/* Overdrive on/off */
-
-#define FL_ROTENA 0x4000	/* Rotary on/off */
-#define FL_ROTSPS 0x8000	/* Rotary speed select */
-
-#define FL_RVBMIX 0x00010000	/* Reverb on/off */
-
-#define FL_DRWRND 0x00020000	/* Randomize drawbars */
-#define FL_KSPLTL 0x00040000	/* Keyboard split point lower/upper */
-
-#define FL_LOWDRW 0x00080000	/* Lower manual drawbars */
-#define FL_PDLDRW 0x00100000	/* Pedal drawbars */
-
-#define FL_KSPLTP 0x00200000	/* Keyboard split point pedal/lower */
-
-#define FL_TRA_PD 0x00400000	/* Transpose for pedal split region */
-#define FL_TRA_LM 0x00800000	/* Transpose for lower split region */
-#define FL_TRA_UM 0x01000000	/* Transpose for upper split region */
-#define FL_TRANSP 0x02000000	/* Global transpose */
-#define FL_TRCH_A 0x04000000	/* Channel A (upper) transpose */
-#define FL_TRCH_B 0x08000000	/* Channel B (lower) transpose */
-#define FL_TRCH_C 0x10000000	/* Channel C (pedal) transpose */
-
-#define FL_VCRUPR 0x20000000	/* Vib/cho upper manual routing */
-#define FL_VCRLWR 0x40000000	/* Vib/cho lower manual routing */
-
 #define ANY_TRSP (FL_TRA_PL | FL_TRA_LM | FL_TRA_UM | FL_TRANSP | \
                   FL_TRCH_A | FL_TRCH_B | FL_TRCH_C)
 
@@ -703,6 +654,18 @@ int bindToProgram (void * pp,
 }
 
 #ifndef PRG_MAIN
+static int format_drawbars(const unsigned int drawbars [], char *buf) {
+  return sprintf (buf, "%c%c%c %c%c%c%c %c%c",
+    '0' + drawbars[0],
+    '0' + drawbars[1],
+    '0' + drawbars[2],
+    '0' + drawbars[3],
+    '0' + drawbars[4],
+    '0' + drawbars[5],
+    '0' + drawbars[6],
+    '0' + drawbars[7],
+    '0' + drawbars[8]);
+}
 /**
  * Installs random values in the supplied array.
  * @param drawbars  Array where to store drawbar settings.
@@ -716,16 +679,7 @@ static void randomizeDrawbars (unsigned int drawbars [], char * buf) {
   }
 
   if (buf != NULL) {
-    sprintf (buf, "%c%c%c %c%c%c%c %c%c",
-	     '0' + drawbars[0],
-	     '0' + drawbars[1],
-	     '0' + drawbars[2],
-	     '0' + drawbars[3],
-	     '0' + drawbars[4],
-	     '0' + drawbars[5],
-	     '0' + drawbars[6],
-	     '0' + drawbars[7],
-	     '0' + drawbars[8]);
+    format_drawbars(drawbars, buf);
   }
 }
 
@@ -998,15 +952,270 @@ void loopProgammes (struct b_programme *p, int all,
   void (*cb)(int, int, const char*, void *), void *arg) {
   int i;
   int cnt=1;
-  for (i=0 ; i < 127; ++i) {
+  for (i=0 ; i < 128; ++i) {
     int pgmNr = i + p->MIDIControllerPgmOffset;
     if (all || p->programmes[pgmNr].flags[0] & FL_INUSE) {
-      cb(cnt++, i+1, p->programmes[pgmNr].name, arg);
+      cb(cnt++, i + p->MIDIControllerPgmOffset, p->programmes[pgmNr].name, arg);
     }
   }
 }
 
 #ifndef PRG_MAIN
+int formatProgram(Programme *p, char *out, int maxlen) {
+  int len = 0;
+  out[0]='\0';
+  if (!(p->flags[0] & FL_INUSE)) {
+    len += snprintf(out, maxlen, " --empty--\n");
+    return len;
+  }
+
+  if (p->flags[0] & FL_DRAWBR) {
+    len += snprintf(out+len, maxlen-len, "U: ");
+    if (p->flags[0] & FL_DRWRND)
+      len += snprintf(out+len, maxlen-len, "-random-");
+    else
+      len += format_drawbars(p->drawbars, out+len);
+    len += snprintf(out+len, maxlen-len, "\n");
+  }
+
+  if (p->flags[0] & FL_LOWDRW) {
+    len += snprintf(out+len, maxlen-len, "L: ");
+    if (p->flags[0] & FL_DRWRND)
+      len += snprintf(out+len, maxlen-len, "-random-");
+    else
+    len += format_drawbars(p->lowerDrawbars, out+len);
+    len += snprintf(out+len, maxlen-len, "\n");
+  }
+
+  if (p->flags[0] & FL_PDLDRW) {
+    len += snprintf(out+len, maxlen-len, "P: ");
+    if (p->flags[0] & FL_DRWRND)
+      len += snprintf(out+len, maxlen-len, "-random-");
+    else
+      len += format_drawbars(p->pedalDrawbars, out+len);
+    len += snprintf(out+len, maxlen-len, "\n");
+  }
+  if (p->flags[0] & (FL_SCANNR|FL_VCRUPR|FL_VCRLWR)) {
+    len += snprintf(out+len, maxlen-len, "vib: ");
+    if (p->flags[0] & FL_SCANNR) {
+      // FL_VCRUPR  FL_VCRLWR
+      int knob = ((p->scanner & 0xf) << 1) - ((p->scanner & CHO_) ? 1 : 2);
+      //len += snprintf(out+len, maxlen-len, "mode: ");
+      switch (knob) {
+	case 0: len += snprintf(out+len, maxlen-len, "v1 "); break;
+	case 1: len += snprintf(out+len, maxlen-len, "c1 "); break;
+	case 2: len += snprintf(out+len, maxlen-len, "v2 "); break;
+	case 3: len += snprintf(out+len, maxlen-len, "c2 "); break;
+	case 4: len += snprintf(out+len, maxlen-len, "v3 "); break;
+	case 5: len += snprintf(out+len, maxlen-len, "c3 "); break;
+	default: len += snprintf(out+len, maxlen-len, "? "); break;
+      }
+    }
+    if (p->flags[0] & FL_VCRUPR) {
+      len += snprintf(out+len, maxlen-len, "uppr: %s ", p->scanner & 0x200 ? "on" : "off");
+    }
+    if (p->flags[0] & FL_VCRLWR) {
+      len += snprintf(out+len, maxlen-len, "lowr: %s ", p->scanner & 0x100 ? "on" : "off");
+    }
+
+    len += snprintf(out+len, maxlen-len, "\n");
+  }
+  if (p->flags[0] & (FL_PRCENA|FL_PRCVOL|FL_PRCSPD|FL_PRCHRM)) {
+    len += snprintf(out+len, maxlen-len, "perc: ");
+    if (p->flags[0] & FL_PRCENA) {
+      len += snprintf(out+len, maxlen-len, "%s ", p->percussionEnabled ? "on" : "off");
+    }
+    if (p->flags[0] & FL_PRCVOL) {
+      len += snprintf(out+len, maxlen-len, "%s ", p->percussionVolume ? "high" : "low");
+    }
+    if (p->flags[0] & FL_PRCSPD) {
+      len += snprintf(out+len, maxlen-len, "%s ", p->percussionSpeed ? "fast" : "slow");
+    }
+    if (p->flags[0] & FL_PRCHRM) {
+      len += snprintf(out+len, maxlen-len, "%s ", p->percussionHarmonic ? "2nd" : "3rd");
+    }
+    len += snprintf(out+len, maxlen-len, "\n");
+  }
+  if (p->flags[0] & FL_OVRSEL) {
+    len += snprintf(out+len, maxlen-len, "overdrive: %s\n", p->overdriveSelect ? "bypass" : "on");
+  }
+  if (p->flags[0] & FL_ROTSPS) {
+    len += snprintf(out+len, maxlen-len, "leslie: ");
+    switch (p->rotarySpeedSelect){
+      case WHIRL_FAST: len += snprintf(out+len, maxlen-len, "fast"); break;
+      case WHIRL_SLOW: len += snprintf(out+len, maxlen-len, "slow"); break;
+      case WHIRL_STOP: len += snprintf(out+len, maxlen-len, "stop"); break;
+      default: len += snprintf(out+len, maxlen-len, "? "); break;
+    }
+    len += snprintf(out+len, maxlen-len, "\n");
+  }
+  if (p->flags[0] & FL_RVBMIX) {
+    len += snprintf(out+len, maxlen-len, "reverb: %d%%\n", (int) rint(100.0*p->reverbMix));
+  }
+  if (p->flags[0] & (FL_KSPLTL|FL_KSPLTP|FL_TRA_PD|FL_TRA_LM|FL_TRA_UM)) {
+    len += snprintf(out+len, maxlen-len, "keyboard-split change\n");
+  }
+  if (p->flags[0] & (FL_TRANSP|FL_TRCH_A|FL_TRCH_B|FL_TRCH_C)) {
+    len += snprintf(out+len, maxlen-len, "transpose..\n");
+  }
+  return len;
+}
+
+static void save_pgm_state_cb(int fnid, const char *key, const char *kv, unsigned char val, void *arg) {
+  Programme * PGM = (Programme*) arg;
+  if      (!strcmp(key, "upper.drawbar16"))  {PGM->drawbars[0] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar513")) {PGM->drawbars[1] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar8"))   {PGM->drawbars[2] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar4"))   {PGM->drawbars[3] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar223")) {PGM->drawbars[4] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar2"))   {PGM->drawbars[5] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar135")) {PGM->drawbars[6] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar113")) {PGM->drawbars[7] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+  else if (!strcmp(key, "upper.drawbar1"))   {PGM->drawbars[8] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_DRAWBR;}
+
+  else if (!strcmp(key, "lower.drawbar16"))  {PGM->lowerDrawbars[0] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar513")) {PGM->lowerDrawbars[1] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar8"))   {PGM->lowerDrawbars[2] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar4"))   {PGM->lowerDrawbars[3] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar223")) {PGM->lowerDrawbars[4] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar2"))   {PGM->lowerDrawbars[5] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar135")) {PGM->lowerDrawbars[6] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar113")) {PGM->lowerDrawbars[7] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+  else if (!strcmp(key, "lower.drawbar1"))   {PGM->lowerDrawbars[8] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_LOWDRW;}
+
+  else if (!strcmp(key, "pedal.drawbar16"))  {PGM->pedalDrawbars[0] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar513")) {PGM->pedalDrawbars[1] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar8"))   {PGM->pedalDrawbars[2] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar4"))   {PGM->pedalDrawbars[3] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar223")) {PGM->pedalDrawbars[4] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar2"))   {PGM->pedalDrawbars[5] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar135")) {PGM->pedalDrawbars[6] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar113")) {PGM->pedalDrawbars[7] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+  else if (!strcmp(key, "pedal.drawbar1"))   {PGM->pedalDrawbars[8] = rint((127-val)*8.0/127.0); PGM->flags[0] |= FL_PDLDRW;}
+
+  else if (!strcmp(key, "percussion.enable"   )) {PGM->percussionEnabled = val > 63 ? TRUE : FALSE; PGM->flags[0] |= FL_PRCENA;}
+  else if (!strcmp(key, "percussion.volume"   )) {PGM->percussionVolume = val/127.0; PGM->flags[0] |= FL_PRCVOL;}
+  else if (!strcmp(key, "percussion.decay"    )) {PGM->percussionSpeed = val > 63 ? TRUE : FALSE; PGM->flags[0] |= FL_PRCSPD;}
+  else if (!strcmp(key, "percussion.harmonic" )) {PGM->percussionHarmonic = val > 63 ? TRUE : FALSE; PGM->flags[0] |= FL_PRCHRM;}
+  else if (!strcmp(key, "overdrive.enable"    )) {PGM->overdriveSelect = val > 63 ? FALSE : TRUE; PGM->flags[0] |= FL_OVRSEL;}
+  else if (!strcmp(key, "reverb.mix"          )) {PGM->reverbMix = val/127.0; PGM->flags[0] |= FL_RVBMIX;}
+  else if (!strcmp(key, "rotary.speed-select" )) {
+    const int hr = (val / 45) % 3; // horn 0:off, 1:chorale  2:tremolo
+    //const int bf = (val / 15) % 3; // drum 0:off, 1:chorale  2:tremolo
+    if (hr == 0) PGM->rotarySpeedSelect = WHIRL_STOP;
+    if (hr == 1) PGM->rotarySpeedSelect = WHIRL_SLOW;
+    if (hr == 2) PGM->rotarySpeedSelect = WHIRL_FAST;
+    PGM->flags[0] |= FL_ROTSPS;
+  }
+  else if (!strcmp(key, "vibrato.routing"     )) {
+    PGM->scanner |= ((val>>5) & 1 ) ?  0x100 : 0; //lower
+    PGM->scanner |= ((val>>5) & 2 ) ?  0x200 : 0; //upper
+    PGM->flags[0] |= FL_VCRUPR | FL_VCRLWR;}
+  else if (!strcmp(key, "vibrato.knob"        )) {
+    int u = val / 23;
+    if (u&1) {
+      PGM->scanner |= CHO_;
+      PGM->scanner |= (u>>1) + 1;
+    } else {
+      PGM->scanner |= (u>>1) + 1;
+    }
+
+    PGM->flags[0] |= FL_SCANNR;}
+}
+
+#include "state.h"
+int saveProgramm(void *instance, int p, char *name, int flagmask) {
+  b_instance * inst = (b_instance*) instance;
+  p += inst->progs->MIDIControllerPgmOffset;
+  if ((p < 0) || (p >= MAXPROGS) || !name) {
+    return -1;
+  }
+  Programme * PGM = &(inst->progs->programmes[p]);
+  memset(PGM, 0, sizeof(Programme));
+  strcat(PGM->name, name);
+  rc_loop_state(inst->state, save_pgm_state_cb, PGM);
+  PGM->flags[0] &= ~flagmask;
+  PGM->flags[0] |= FL_INUSE;
+#if 0
+  char tmp[256];
+  formatProgram(PGM, tmp, 256);
+  printf("SAVED STATE: %s\n", tmp);
+#endif
+  return 0;
+}
+
+void writeProgramm(int pgmNr, Programme *p, const char *sep, FILE * fp) {
+  char tmp[24];
+  fprintf(fp, "%d {%s  name=\"%s\"", pgmNr, sep, p->name);
+  if ((p->flags[0] & FL_DRAWBR) && !(p->flags[0] & FL_DRWRND)) {
+    format_drawbars(p->drawbars, tmp);
+    fprintf(fp, "%s, drawbarsupper=\"%s\"", sep, tmp);
+  }
+  if ((p->flags[0] & FL_LOWDRW) && !(p->flags[0] & FL_DRWRND)) {
+    format_drawbars(p->lowerDrawbars, tmp);
+    fprintf(fp, "%s, drawbarslower=\"%s\"", sep, tmp);
+  }
+  if ((p->flags[0] & FL_PDLDRW) && !(p->flags[0] & FL_DRWRND)) {
+    format_drawbars(p->pedalDrawbars, tmp);
+    fprintf(fp, "%s, drawbarspedals=\"%s\"", sep, tmp);
+  }
+  if (p->flags[0] & FL_SCANNR) {
+    int knob = ((p->scanner & 0xf) << 1) - ((p->scanner & CHO_) ? 1 : 2);
+    fprintf(fp, "%s, vibrato=", sep);
+    switch (knob) {
+      case 0: fprintf(fp, "v1"); break;
+      case 1: fprintf(fp, "c1"); break;
+      case 2: fprintf(fp, "v2"); break;
+      case 3: fprintf(fp, "c2"); break;
+      case 4: fprintf(fp, "v3"); break;
+      case 5: fprintf(fp, "c3"); break;
+      default: break;  // XXX
+    }
+  }
+  if (p->flags[0] & FL_VCRUPR) { fprintf(fp, "%s, vibratoupper=%s", sep, (p->scanner&0x200) ? "on" : "off"); }
+  if (p->flags[0] & FL_VCRLWR) { fprintf(fp, "%s, vibratolower=%s", sep, (p->scanner&0x100) ? "on" : "off"); }
+  if (p->flags[0] & FL_PRCENA) { fprintf(fp, "%s, perc=%s", sep, p->percussionEnabled ? "on" : "off"); }
+  if (p->flags[0] & FL_PRCVOL) { fprintf(fp, "%s, percvol=%s", sep, p->percussionVolume ? "soft" : "normal"); }
+  if (p->flags[0] & FL_PRCSPD) { fprintf(fp, "%s, percspeed=%s", sep, p->percussionSpeed ? "fast" : "slow"); }
+  if (p->flags[0] & FL_PRCHRM) { fprintf(fp, "%s, percharm=%s", sep, p->percussionHarmonic ? "2nd" : "3rd"); }
+  if (p->flags[0] & FL_OVRSEL) { fprintf(fp, "%s, overdrive=%s", sep, p->overdriveSelect ? "off" : "on"); }
+  if (p->flags[0] & FL_RVBMIX) { fprintf(fp, "%s, reverbmix=%f", sep, p->reverbMix); }
+  if (p->flags[0] & FL_ROTSPS) {
+    fprintf(fp, "%s, rotaryspeed=", sep);
+    switch(p->rotarySpeedSelect) {
+      case WHIRL_FAST: fprintf(fp , "fast"); break;
+      case WHIRL_SLOW: fprintf(fp , "slow"); break;
+      case WHIRL_STOP: fprintf(fp , "stop"); break;
+      default: break;  // XXX
+    }
+  }
+
+  if (p->flags[0] & FL_KSPLTL) { fprintf(fp, "%s, keysplitlower=%d", sep, p->keyboardSplitLower); }
+  if (p->flags[0] & FL_KSPLTP) { fprintf(fp, "%s, keysplitpedals=%d", sep, p->keyboardSplitPedals); }
+
+  if (p->flags[0] & FL_TRANSP) { fprintf(fp, "%s, transpose=%d", sep, p->transpose[TR_TRANSP]); }
+  if (p->flags[0] & FL_TRCH_A) { fprintf(fp, "%s, transposeupper=%d", sep, p->transpose[TR_CHNL_A]); }
+  if (p->flags[0] & FL_TRCH_B) { fprintf(fp, "%s, transposelower=%d", sep, p->transpose[TR_CHNL_B]); }
+  if (p->flags[0] & FL_TRCH_C) { fprintf(fp, "%s, transposepedals=%d", sep, p->transpose[TR_CHNL_C]); }
+
+  if (p->flags[0] & FL_TRA_PD) { fprintf(fp, "%s, trssplitpedals=%d", sep, p->transpose[TR_CHA_PD]); }
+  if (p->flags[0] & FL_TRA_LM) { fprintf(fp, "%s, trssplitlower=%d", sep, p->transpose[TR_CHA_LM]); }
+  if (p->flags[0] & FL_TRA_UM) { fprintf(fp, "%s, trssplitupper=%d", sep, p->transpose[TR_CHA_UM]); }
+
+  fprintf(fp, "%s}\n",sep);
+}
+
+void exportProgramms(struct b_programme *pgm, FILE * fp) {
+  int i;
+  for (i=0 ; i < 128; ++i) {
+    int pgmNr = i + pgm->MIDIControllerPgmOffset;
+    if (!(pgm->programmes[pgmNr].flags[0] & FL_INUSE)) {
+      continue;
+    }
+    writeProgramm(pgmNr, &pgm->programmes[pgmNr], "\n    ", fp);
+  }
+}
+
 struct b_programme *allocProgs() {
   struct b_programme *p = (struct b_programme*) calloc(1, sizeof(struct b_programme));
   if (!p) return NULL;
